#!/bin/sh

# rescue-build.sh (W) 2010 by peter@holik.at

# suggest: dpkg -P brltty brltty-x11 mdadm sfdisk bzip2 whiptail eject syslinux

IMAGEDIR=image
INITRAMFSDIR=initramfs

mkdir -p ${IMAGEDIR}

# magic file to find blockdevice with backup
touch ${IMAGEDIR}/.initramfs-backup

mkdir -p ${INITRAMFSDIR}

cat >${INITRAMFSDIR}/initramfs.conf <<EOF
MODULES=most
BUSYBOX=y
BOOT=install
verbose=n
EOF

cat >${INITRAMFSDIR}/update-initramfs.conf <<EOF
update_initramfs=yes
backup_initramfs=no
EOF

touch ${INITRAMFSDIR}/modules

mkdir -p ${INITRAMFSDIR}/hooks

# include filesed needed to restore
cat >${INITRAMFSDIR}/hooks/includes <<EOF
#!/bin/sh

PREREQ=""

prereqs()
{
        echo "\$PREREQ"
}
case \$1 in
# get pre-requisites
prereqs)
        prereqs
        exit 0
        ;;
esac

# Hooks for loading extra kernel bits into the initramfs

. /usr/share/initramfs-tools/hook-functions

copy_exec /bin/tar /bin
copy_exec /bin/bzip2 /bin
copy_exec /sbin/sfdisk /bin
copy_exec /sbin/mkfs.ext2 /bin
copy_exec /sbin/mkfs.ext3 /bin
copy_exec /sbin/mkfs.ext4 /bin
copy_exec /sbin/tune2fs /bin
copy_exec /sbin/mdadm /bin
copy_exec /usr/bin/find /bin
copy_exec /usr/bin/clear /bin
copy_exec /usr/bin/whiptail /bin
copy_exec /usr/bin/eject /bin

mkdir -p \${DESTDIR}/.terminfo/l
cp /usr/share/terminfo/l/linux \${DESTDIR}/.terminfo/l
cp /usr/lib/libfribidi.so.0 \${DESTDIR}/usr/lib
EOF
chmod 755 ${INITRAMFSDIR}/hooks/includes

mkdir -p ${INITRAMFSDIR}/scripts

cat >${INITRAMFSDIR}/scripts/install <<EOF
backup_mountpoint="/backup"
disk_mountpoint="/disk"
backup_device=""

do_halt()
{
    sync

    umount \${backup_mountpoint} >/dev/null

    eject -p -m \${backup_device} >/dev/null 2>&1

    read -p 'Please remove the stick and press ENTER:' x

    reboot
}

do_error()
{
    [ -z "\$1" ] && panic failed || echo \$1
    do_halt
}

do_result()
{
    [ \$1 -eq 0 ] && echo ok || do_error
}

find_backupfs()
{
    for sysblock in \$(echo /sys/block/* | tr ' ' '\n' | grep -vE "/(loop|ram|dm-|fd)")
    do
# for testing in kvm disabled
#        if [ "\$(cat \${sysblock}/removable)" = "1" ]
#        then
            for part in "\${sysblock}" "\${sysblock}"/*
            do
                if [ -e "\${part}/dev" ]
                then
                    mount -n -t ext2 -o ro "/dev/\${part##*/}" \${backup_mountpoint} >/dev/null 2>&1 || continue
                    if [ -f \${backup_mountpoint}/.initramfs-backup ]
                    then
                        backup_device="/dev/\${part##*/}"
                        return 0
                    else
                        umount \${backup_mountpoint} >/dev/null
                    fi
                fi
            done
#        fi
    done

    return 1
}

get_disk()
{
    local sysblock
    local disk

    for sysblock in /sys/block/[vsh]d[a-d]
    do
            [ -d \$sysblock ] && if [ "\$(cat \$sysblock/removable)" = "0" ]; then
                disk="/dev/\${sysblock##*/}"
                [ -n "\$1" ] && [ "\$1" = "\$disk" ] && continue
                echo "\$disk"
                return 0
            fi
    done

    return 1
}

do_install()
{
    clear

    local DISK2
    local DISK1=\$(get_disk)
    [ -z "\${DISK1}" ] && do_error 'No Harddisk found'

    [ -z "\$RAID_UUIDS" ] || {
        DISK2=\$(get_disk \${DISK1})
        [ -z "\${DISK2}" ] && do_error 'No second Harddisk found'
    }
    DISK=\${DISK1}

    IFS=' '

    echo \${DISK_PARTITIONS} | while read PARTITIONS; do
        [ -n "\${PARTITIONS}" ] && {
            echo -n  "Partitioning Harddisk \${DISK}..."
            echo \${PARTITIONS} | sed "s/;/\n/g" | sfdisk -q -uM -L \${DISK} >/dev/null 2>&1
            do_result \$?
            DISK=\${DISK2}
        }
    done

    raidnum=0
    echo \${RAID_UUIDS} | while read UUID; do
        [ -n "\${UUID}" ] && {
            echo -n "Create Raid md\${raidnum} on \${DISK1}\$((raidnum+1)) \${DISK2}\$((raidnum+1))..."
            mdadm --create /dev/md\${raidnum} --level=1 --raid-devices=2 --uuid=\${UUID} \${DISK1}\$((raidnum+1)) \${DISK2}\$((raidnum+1)) >/dev/null 2>&1
            do_result \$?
            raidnum=\$((raidnum+1))
        }
    done

    ln -s /proc/mounts /etc/mtab

    [ -n "\$RAID_UUIDS" ] && DISK=/dev/md

    echo -e \${ARCHIVES} | while read PARTITION UUID FSTYPE ARCHIVE; do
        echo -n "Create \${DISK}\${PARTITION} filesystem (\${FSTYPE})..."
        [ -n "\${GRUB1_INODESIZE}" ] && INODESIZEPARAM="-I \${GRUB1_INODESIZE}"
        mkfs.\${FSTYPE} -q \${INODESIZEPARAM} \${DISK}\${PARTITION}
        do_result \$?
        echo -n "Set \${DISK}\${PARTITION} UUID to \${UUID}..."
        tune2fs -U \${UUID} \${DISK}\${PARTITION} >/dev/null 2>&1
        do_result \$?
        [ -n "\${ARCHIVE}" ] && {
            modprobe -q \${FSTYPE}
            echo -n "Mount \${DISK}\${PARTITION}..."
            mount -n -t \${FSTYPE} \${DISK}\${PARTITION} \${disk_mountpoint}
            do_result \$?
            echo -n 'Copy Files...'
            tar -xSp --atime-preserve -f \${backup_mountpoint}/\${ARCHIVE} --directory \${disk_mountpoint}
            do_result \$?
        }
        echo "\${ARCHIVE}" | grep -q root && {
            mount -n -o bind /dev \${disk_mountpoint}/dev
            mount -n -o bind /proc \${disk_mountpoint}/proc

            echo -e \${SWAP} | while read PARTITION UUID; do
                [ -n "\${PARTITION}" ] && {
                    echo -n 'Create Swap...'
                    chroot \${disk_mountpoint} /sbin/mkswap -U \${UUID} \${DISK}\${PARTITION} >/dev/null
                    do_result \$?
                }
            done

            sync

            echo -n 'Install Bootloader...'

            if [ \${GRUB} -eq 1 ]; then
                chroot \${disk_mountpoint} /usr/sbin/grub --batch <<EOT >/dev/null 2>&1
device (hd0) \${DISK1}
root (hd0,\${GRUB1_PARTITION})
setup (hd0)
quit
EOT
            elif [ \${GRUB} -eq 2 ]; then
                chroot \${disk_mountpoint} /usr/sbin/grub-setup --force \${DISK1} >/dev/null 2>&1
            else
                do_error 'bootloader undefined'
            fi
            do_result \$?

            [ -n "\${DISK2}" ] && if [ \${GRUB} -eq 1 ]; then
                chroot \${disk_mountpoint} /usr/sbin/grub --batch <<EOT >/dev/null 2>&1
device (hd0) \${DISK2}
root (hd0,\${GRUB1_PARTITION})
setup (hd0)
quit
EOT
            elif [ \${GRUB} -eq 2 ]; then
                chroot \${disk_mountpoint} /usr/sbin/grub-setup --force \${DISK2} >/dev/null 2>&1
            else
                do_error 'bootloader undefined'
            fi
            umount \${disk_mountpoint}/proc
            umount \${disk_mountpoint}/dev

            echo -e \${MKDIRS} | while read DIR; do
                mkdir -p \${disk_mountpoint}\${DIR}
            done
        }
        [ -n "\${ARCHIVE}" ] && umount \${disk_mountpoint}
    done
}

mountroot()
{
    clear
    echo -n "Search for Backupdevice"

    modprobe virtio_blk

    mkdir -p \${backup_mountpoint}
    mkdir -p \${disk_mountpoint}

    # Scan local devices for the image
    for i in 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19
    do
            find_backupfs && break
            echo -n "."
            sleep 1
    done

    do_result \$?

    for ARCHIVE in \${backup_mountpoint}/*.cfg; do item=\${ARCHIVE##*/}; items="\${items} \${item%%\.*} 0"; done

    rm /dev/tty
    ln -s /dev/console /dev/tty
    while ! host=\$(whiptail --nocancel --noitem --menu "Choose host" 15 19 5 \${items} 2>&1 >/dev/tty); do true; done;

    . \${backup_mountpoint}/\${host}.cfg

    do_install

    do_halt
}
EOF

# generate initrd
mkinitramfs -d $(pwd)/${INITRAMFSDIR} -o ${IMAGEDIR}/initrd.img

# copy kernel to imagedir
cp /boot/vmlinuz-$(uname -r) ${IMAGEDIR}/vmlinuz

for ARCHIVE in *.cfg; do ln -f ${ARCHIVE%%\.*}.* "${IMAGEDIR}"; done
